from math import pi
cdef double PI = pi

# Cython as of 0.98.1 does not support complex numbers (AFAIK) so this
# code does not use them which is why this is different from the pure
# Python version.

cdef class Vortex:
    # Position of the vortex.
    cdef public double x, y
    # Strength of the vortex.
    cdef public double strength
    # The velocity components of the vortex.
    cdef public double u, v

    def __init__(self, double x=0.0, double y=0.0, double strength=1.0):
        self.x = x
        self.y = y
        self.strength = strength
        self.u = self.v = 0.0

    cdef void _eval_velocity(self, double x, double y, double *vx, double *vy):
        """Evaluate the velocity of the vortex at a given position.
        
        This function will be blazing fast since it is effectively
        raw C.  The only trouble is you can't call this from Python.

        Parameters:
        -----------
        x, y : position to evaluate velocity at.
        vx, vy : velocities to compute and set.
        """
        cdef double dx = x - self.x
        cdef double dy = y - self.y
        cdef double r2 = dx*dx + dy*dy
        cdef double fac = -self.strength/(2*PI*r2)
        vx[0], vy[0] = fac*dy, fac*dx

    def eval_velocity(self, double x, double y):
        """Computes the velocity and returns a tuple.  Callable from
        Python.
        """
        cdef double vx, vy
        self._eval_velocity(x, y, &vx, &vy)
        return vx, vy


cdef class VortexManager:

    # A list of vortices we manage.
    cdef list vortices

    def __init__(self, vortices=None):
        self.vortices = []
        if vortices is not None:
            self.vortices = vortices

    def get(self, int i):
        return self.vortices[i]

    def set(self, pos, str):
        self.vortices = []
        cdef int i, N = len(pos)
        for i in range(N):
            p = pos[i]
            v = Vortex(p.real, p.imag, str[i])
            self.vortices.append(v)

    def eval_velocity(self, double x, double y):
        """Evaluate the velocity of the vortices at the given position.
        """
        cdef double u, v, vx = 0.0, vy = 0.0
        cdef Vortex vor
        cdef int i, N = len(self.vortices)
        for i in range(N):
            vor = self.vortices[i]
            vor._eval_velocity(x, y, &u, &v)
            vx += u
            vy += v
        return vx, vy

    def velocity(self):
        """Velocity of the vortices on themselves."""
        cdef double vx, vy, u, v
        cdef Vortex va, vb
        cdef int i, j, N = len(self.vortices)
        for i in range(N):
            va = self.vortices[i]
            vx = vy = 0.0
            for j in range(N):
                vb = self.vortices[j]
                if i != j:
                    vb._eval_velocity(va.x, va.y, &u, &v)
                    vx += u
                    vy += v
            va.u = vx
            va.v = vy

